package xelfi.awt;

import java.awt.*;

/**
 * Any object that wants to be scrolled and react 
 * to events in scrollable owner should implement this 
 * interface.

 * @version 1.00, 1 Feb 1997
 * @author Dafe
	Last change:  DF    2 Feb 97    3:43 pm
*/
public interface XelfiScrollable {

	/**
	 * Sets the upper left point of the view.
	*/
	public void setView (int x, int y);

	/**
	 * Returns upper left corner of the view.
	*/
	public Point getView ();

	/**
	 * Paints specified area (absolute coordinates - it means not relative
	 * to getView() point).
	 * clearBackgrouns says whether object is responsible for clearing
	 * background in area.
	*/
	public void paintArea (Graphics g, int x, int y,
												 int width, int height, boolean clearBackground);

	/**
	 * Returns dimension of curently visible area.
	*/
	public Dimension visibleSize ();

	/**
	 * Returns dimension needed for all scrollable
	 * object screen area to be shown.
	*/
	public Dimension wholeSize ();

	/**
	 * Sets scrollable owner - object which holds scrollbars
	 * and should be notified when changes in coordinates of
	 * visible area are made.
	*/
	public void setScrollableOwner (XelfiScrollableOwner scrollableOwner);

	/**
	 * Returns scrollable owner - object which usually holds scrollbars
	*/
	public XelfiScrollableOwner getScrollableOwner ();

}


