
package xelfi.browser.document;

import java.util.*;
import java.io.*;
import xelfi.editor.Source;
import xelfi.editor.Document;
import xelfi.editor.Position;

import xelfi.top.TopLevel;
import xelfi.compiler.Output;

//import xelfi.browser.util.PositionedInputStream;

class Parser extends Thread
{

    // DATA:
    static final String        java        = ".java";
    static final String        form        = ".form";


    // Nasledujici metody se pouzivaji na naparsovani
    // objektu a memberu jednoko package...

    private static Modifiers readModifiers(
        XStreamTokenizer st) throws IOException
    {
        //System.out.println("readModifiers");
        int ch = 0;
        Modifiers modifiers = new Modifiers();

        ch = st.nextToken();
        while( ch == XStreamTokenizer.TT_WORD )
        {
            if( st.sval.equals("final") )
                modifiers.setFinal();
            else
            if( st.sval.equals("static") )
                modifiers.setStatic();
            else
            if( st.sval.equals("native") )
                modifiers.setNative();
            else
            if( st.sval.equals("synchronized") )
                modifiers.setSynchronized();
            else
            if( st.sval.equals("abstract") )
                modifiers.setAbstract();
            else
            if( st.sval.equals("threadsafe") )
                modifiers.setThreadsafe();
            else
            if( st.sval.equals("transient") )
                modifiers.setTransient();
            else
            if( st.sval.equals("public") )
                modifiers.setPublic();
            else
            if( st.sval.equals("private") )
                modifiers.setPrivate();
            else
            if( st.sval.equals("protected") )
                modifiers.setProtected();
            else
                break;
            ch = st.nextToken();
        }//while
        st.pushBack();

        //System.out.println("readModifiers end");
        return modifiers;
    }
    /**
     * This method reads xxx.xxx.xxx from
     * @param st XStreamTokenizer.
     * @param star if star == true, read xx.xxx.xx.* too
     * @return readed text without whitespaces or null
     *   if error occurred
     */
    private static String readDotName(XStreamTokenizer st,boolean star)
        throws IOException
    {
        StringBuffer sb = new StringBuffer();
        int ch = st.nextToken();
        while( ch == XStreamTokenizer.TT_WORD )
        {
            sb.append(st.sval);
            ch = st.nextToken();
            if( ch == '.' )
            {
                sb.append('.');
                ch = st.nextToken();
                continue;
            }
            st.pushBack();
            return new String(sb);
        }
        if( star && (ch == '*') )
        {
            sb.append('*');
            return new String(sb);
        }
        return null;
    }
    /**
     * This method reads xxx.xxx ,xx.xxx.xx , xx from
     * @param st    XStreamTokenizer.
     * @param star  if star == true, read xx.xxx.xx.* , ... too
     * @param names readed text without whitespaces
     * @return      == false if an error occurred
     */
    private static String[] readDotNames(XStreamTokenizer st,boolean star)
        throws IOException
    {
        String s;
        Vector v = new Vector(10,10);
        String[] names;
        int ch;
        do
        {
            s = readDotName(st,star);
            if( s == null )
            {
                int i,k = v.size();
                names = new String[k];
                for(i=0;i<k;i++)
                    names[i] = (String)v.elementAt(i);
                return null;
            }
            v.addElement(s);
            ch = st.nextToken();
        }while( ch == ',' );

        st.pushBack();
        int i,k = v.size();
        names = new String[k];
        for(i=0;i<k;i++)
            names[i] = (String)v.elementAt(i);
        return names;
    }
    /**
     * This method reads [][][] from
     * @param st    XStreamTokenizer.
     * @return      readed text without whitespaces or null
     *              if an error occurred
     */
    private static String readII(XStreamTokenizer st)
        throws IOException
    {
        StringBuffer sb = new StringBuffer();
        int ch = st.nextToken();
        while( ch == '[' )
        {
            sb.append('[');
            ch = st.nextToken();
            if( ch != ']' )
            {
                st.pushBack();
                return null;
            }
            sb.append(']');
            ch = st.nextToken();
        }
        st.pushBack();
        return new String(sb);
    }
    private static void readBody(XStreamTokenizer st) throws IOException
    {   // cte xxx ;
        // cte xxx { xxx{xx}xx}

        int ch;
        do{
            ch = st.nextToken();
        }while( ch != '{' && ch != ';' && ch != XStreamTokenizer.TT_EOF );
        if( ch != '{' )
            return;
        int depth = 1;
        do{
            ch = st.nextToken();
            if( ch == '{' ) depth ++;
            else
            if( ch == '}' ) depth --;
            else
            if( ch == XStreamTokenizer.TT_EOF )
                return;
        }while( depth > 0 );
        ch = st.nextToken();
        if( ch != ';' ) st.pushBack();

        return;
    }
    private static BrMember  readMembers1(
                            MyPosition positionB,
                            MyPosition commentPositionB,
                            MyPosition commentPositionE,
                            BrObject comp,
                            XStreamTokenizer st,
                            XFile pXFile,
                            Modifiers modifiers,
                            String type) throws IOException
    {   // cte variable a metody...
        // v pType mam typ xxx.xxxx.xxx
        // precetl jsem:  modifiers + xxx.xxx.xxx

        //System.out.println("readMembers1");
        int ch = 0;
        String name = "";
        String varInit = "",s;
        String[] ss = null;
        boolean one = true;
        BrMember lastMember = null;

        s = readII(st);
        if( s != null )
            type = type + s;
        // eror is ignored

        MyPosition pomPosition = st.getMiddle();
        // zmenili jsme bude fungovat ??? !!!!

        // name
        ch = st.nextToken();

        if( ch != XStreamTokenizer.TT_WORD )
        {
            TopLevel.getOutput().println("Name of method/variable " +
                "expected at l." + st.lineno());
            st.pushBack();
            return null;
        }else
        {
            // znova precet jmeno
            name = st.sval;
            ch = st.nextToken();
            if( (ch != ';') && (ch != '=') &&
                (ch != '(') && (ch != '[') && (ch != ',') )
            {
                TopLevel.getOutput().println("Error behind name: " +
                    name + " l." + st.lineno());
                st.pushBack();
                return null;
            }
            if( ch == '(' )
            {// method
                st.readTo(')',')');
                varInit = '(' + st.sval + ')';

                // throws
                ch = st.nextToken();
                if( (ch == XStreamTokenizer.TT_WORD) &&
                    st.sval.equals("throws") )
                {
                    if( (ss = readDotNames(st,false)) == null )
                    {
                        TopLevel.getOutput().println("Error in throws " +
                            "at l." + st.lineno());
                        ss = new String[1];
                        ss[0] = "???";
                    }
                    ch = st.nextToken();
                }
                if( (ch != ';') && (ch != '{') )
                {
                    TopLevel.getOutput().println("Error in method definition."+
                        "\';' or '{' expected at l." + st.lineno());
                    st.pushBack();
                    return null;
                }

                comp.addComponent(
                    lastMember = new BrMethod(
                                    positionB,
                                    st.getPosition(),
                                    commentPositionB,
                                    commentPositionE,
                                    comp,
                                    name,
                                    pXFile,
                                    modifiers,
                                    type,
                                    varInit,
                                    ss));
                st.pushBack();

            }else
            {// variable
                // jsem za = ; , [
                while( true )
                {
                    st.pushBack();
                    s = readII(st);
                    if( s != null )
                        name = name + s;
                    //error is ignored!!!

                    ch = st.nextToken();

                    if( ch == '=' )
                    {
                        ch = st.readTo(';',',');
                        varInit = st.sval;
                    }else
                        varInit = "";

                    if( ch != ';' )
                    {
                        one = false;
                    }

                    comp.addComponent(
                        lastMember = new BrVariable(
                            one ? positionB : pomPosition,
                            one ? st.getPosition() : st.getPreviousEnd(),
                            commentPositionB,
                            commentPositionE,
                            one,comp,name,pXFile,
                            modifiers,type,varInit));

                    if( ch == ';' )
                    {
                        st.pushBack();
                        break;
                    }
                    //st.nextToken();// prectu znovu ,

                    if( ch != ',' )
                    {
                        TopLevel.getOutput().println("Error behind " +
                            "definition of variable at l." + st.lineno());
                        st.pushBack();
                        return null;
                    }

                    pomPosition = st.getNextBegin();

                    // name
                    ch = st.nextToken();
                    if( ch != XStreamTokenizer.TT_WORD )
                    {
                        TopLevel.getOutput().println("Name of variable " +
                            "expected at l." + st.lineno());
                        st.pushBack();
                        return null;
                    }
                    name = st.sval;
                    ch = st.nextToken();
                }// while(true)
            }//else variable
        }//else je WORD
        //System.out.println("readMembers1 end");
        return lastMember;
    }

    private static void readMembers(
                        BrObject comp,
                        XStreamTokenizer st,
                        XFile pXFile) throws IOException
    { // Cte membery: static a constructory, ostatni predava readMembers1

        //System.out.println("readMembers");
        int ch = 0;
        String name;
        String[] ss = null;
        Modifiers modifiers;
        MyPosition position;
        MyPosition commentPositionB = null;
        MyPosition commentPositionE = null;
        BrMember lastMember = null;
        do
        {
            position = st.getNextBegin();
            commentPositionB = st.getCommentPositionBegin();
            if( commentPositionB != null )
                commentPositionE = st.getCommentPositionEnd();

            // Read MODIFIERS
            modifiers = readModifiers(st);

            // Read type??? name of constructor??? {???
            ch = st.nextToken();

            // je to EOF, {, WORD (constructor,readMembers1),
            if( ch == XStreamTokenizer.TT_EOF )
                return;
            if( (ch == XStreamTokenizer.TT_WORD) &&
                (st.sval.equals("throws")) && modifiers.staticM() )
            {// static
                // muze mit static throw ???
                if( (ss = readDotNames(st,false)) == null )
                {
                    TopLevel.getOutput().println("Error in throws " +
                        "at l." + st.lineno());
                    ss = new String[1];
                    ss[0] = "???";
                }
                ch = st.nextToken();
                if( (ch != ';') && (ch != '{') )
                {
                    TopLevel.getOutput().println("Error in static method " +
                        "definition. ';' or '{' expected at l." + st.lineno());
                    break;
                }
                comp.addComponent(lastMember =
                    new BrMethod(position,st.getPosition(),
                    commentPositionB,commentPositionE,
                    comp,"static",pXFile,modifiers,
                    "","",ss));
                st.pushBack();
            }else
            if( (ch == '{') && modifiers.staticM() )
            {// static
                comp.addComponent(lastMember =
                    new BrMethod(position,st.getPosition(),
                    commentPositionB,commentPositionE,
                    comp,"static",pXFile,modifiers,
                    "","",""));
                st.pushBack();
            }else
            if( ch == XStreamTokenizer.TT_WORD )
            {
                st.pushBack();
                // ctu typ

                name = readDotName(st,false);
                if( name == null )
                {
                    TopLevel.getOutput().println("Type definition " +
                        "expected at l." + st.lineno());
                    break;
                }

                ch = st.nextToken();
                if( ch != '(' )
                {// ctu normalni member ne static ani constructor !
                    st.pushBack();
                    lastMember = readMembers1(position,commentPositionB,
                        commentPositionE,comp,st,pXFile,modifiers,name);
                }else
                {// constructor

                    st.readTo(')',')');
                    String init = '(' + st.sval + ')';

                    //cti throws ???
                    ch = st.nextToken();
                    if( (ch == XStreamTokenizer.TT_WORD) &&
                        st.sval.equals("throws") )
                    {
                        if( (ss = readDotNames(st,false)) == null )
                        {
                            TopLevel.getOutput().println("Error in throws " +
                                " in constructor at l." + st.lineno());
                            ss = new String[1];
                            ss[0] = "???";
                        }
                        ch = st.nextToken();
                    }

                    if( (ch != ';') && (ch != '{') )
                    {
                        TopLevel.getOutput().println("Error in constructor " +
                            "definition. ';' or '{' expected at l." + st.lineno());
                        break;
                    }

                    comp.addComponent(lastMember = new BrMethod(position,
                        st.getPosition(),commentPositionB,
                        commentPositionE,
                        comp,name,pXFile,
                        modifiers,"",init,ss));

                    st.pushBack();
                }
            }else // nenasel jmeno (== WORD)
            if( ch == '}' )
            {
                ch = st.nextToken();
                if( ch != ';' )
                    st.pushBack();
                return;
            }else // neni WORD ani { ani }
            {
                TopLevel.getOutput().println("Error in type definition " +
                    " of method/variable at l." + st.lineno());
                //return;
            }

            //po uspesne nactenem constructoru, static
            // ci po ukonceni readMembers1
            readBody(st);
            if( lastMember != null )
                lastMember.setBodyEnd(st.getPosition());
            //ch = st.nextToken();
        }while( true );

        //System.out.println("readMembers end");
    }
    private static void readObject(
                        MyPosition position,
                        MyPosition commentPositionB,
                        MyPosition commentPositionE,
                        BrPackage pkg,
                        XStreamTokenizer st,
                        XFile xf,
                        Modifiers modifiers,
                        boolean cls,
                        boolean form,
                        boolean loaded)
                throws IOException
    {   // cte za: "modifiers.. class"


        //System.out.println("readObject");
        int ch;
        String name = "";
        String extendsM = "";
        BrObject comp = null;
        String[] implementsM = new String[0];

        // ctu jmeno obj.
        ch = st.nextToken();
        if( ch != XStreamTokenizer.TT_WORD )
        {
            TopLevel.getOutput().println("Name of class/interface " +
                "expected at l." + st.lineno());
            st.pushBack();
            return;
        }
        name = st.sval;

        // extends or implements???
        ch = st.nextToken();
        if( ch == XStreamTokenizer.TT_WORD )
        {
            if( st.sval.equals("extends") && cls )
            {
                ch = st.nextToken();
                if( (ch == XStreamTokenizer.TT_WORD) &&
                    !st.sval.equals("implements") )
                {
                    st.pushBack();
                    extendsM = readDotName(st,false);
                    if( extendsM == null )
                    {
                        TopLevel.getOutput().println("Error in extends" +
                            " at l." + st.lineno());
                        extendsM = "";
                    }
                    ch = st.nextToken();
                }else
                    st.pushBack();
            }
            if( ch == XStreamTokenizer.TT_WORD )
            {
                if( (st.sval.equals("implements") && cls) ||
                    (st.sval.equals("extends") && !cls) )
                {
                    String s = st.sval;
                    if( (implementsM = readDotNames(st,false)) == null )
                    {
                        TopLevel.getOutput().println("Error in " + s +
                            " at l." + st.lineno());
                        implementsM = new String[1];
                        implementsM[0] = "???";
                    }
                }
            }else // neni to ani implements ani extends
                 st.pushBack();
        }else // za name neni WORD
            st.pushBack();

        //prevod Implements
        /*int i,k = implementsM.size();
        implementsMM = new String[k];
        for(i=0;i<k;i++)
            implementsMM[i] = (String)implementsM.elementAt(i);
        implementsM.removeAllElements();*/

        // ctu za implements ...
        ch = st.nextToken();
        while( ch != '{' )
        {
            TopLevel.getOutput().println("Error in definiton of " +
                (cls ? "class" : "interface") +
                ". '{' expected at l." + st.lineno());
            ch = st.nextToken();
        }

        if( cls )
            pkg.addComponent(
                comp = new BrClass(
                            position,
                            st.getPosition(),
                            commentPositionB,
                            commentPositionE,
                            pkg,
                            name,
                            xf,
                            modifiers,
                            extendsM,
                            implementsM,
                            form));
        else
            pkg.addComponent(
                comp = new BrInterface(
                            position,
                            st.getPosition(),
                            commentPositionB,
                            commentPositionE,
                            pkg,
                            name,
                            xf,
                            modifiers,
                            implementsM));

        comp.isLoaded = loaded;
        readMembers(comp,st,xf);
        comp.setBodyEnd(st.getPosition());
        //System.out.println("readObject end");
        return;
    }
    static void reparseFile(BrPackage pkg,XFile xf,boolean form)
        throws FileNotFoundException,IOException
    {   // cte Modifiers class   a  Modifiers interface

        //System.out.println("reparse file: " + xf.getPackageAndName());
        TopLevel.getOutput().println("reparse file: " +
            xf.getPackageAndName());

        XStreamTokenizer st;
        int i,k = sources.size();
        Source s = null;
        boolean loaded = false;
        for(i=0;i<k;i++)
            if( ((XFile)(s = (Source)sources.elementAt(i)).
                    getFile()).equals(xf) )
                break;
        if( i<k )
        {
            Document d = s.getDocument();
            Position p = d.getBegin();
            st = new XStreamTokenizer(p.getInputStream());
            loaded = true;
        }else
        {
            st = new XStreamTokenizer(new FileInputStream(xf.getFile()));
        }

        st.commentReport(false);
        int ch;
        Modifiers modifiers = new Modifiers();
        MyPosition position = null;
        MyPosition commentPositionB = null;
        MyPosition commentPositionE = null;
        boolean In = false;

//        position = st.getNextBegin();
        while( true )
        {
            ch = st.nextToken();
            switch( ch )
            {
            case XStreamTokenizer.TT_EOF:
                //System.out.println("reparseFile end");
                ComponentManager.reparseTimer();
                return;
            case XStreamTokenizer.TT_WORD:
                if( st.sval.equals("abstract") )
                {
                    modifiers.setAbstract();
                    if( !In )
                    {
                        position = st.getMiddle();
                        commentPositionB = st.getCommentPositionBegin();
                        if( commentPositionB != null )
                            commentPositionE = st.getCommentPositionEnd();
                        In = true;
                    }
                }else
                if( st.sval.equals("final") )
                {
                    modifiers.setFinal();
                    if( !In )
                    {
                        position = st.getMiddle();
                        commentPositionB = st.getCommentPositionBegin();
                        if( commentPositionB != null )
                            commentPositionE = st.getCommentPositionEnd();
                        In = true;
                    }
                }else
                if( st.sval.equals("public") )
                {
                    modifiers.setPublic();
                    if( !In )
                    {
                        position = st.getMiddle();
                        commentPositionB = st.getCommentPositionBegin();
                        if( commentPositionB != null )
                            commentPositionE = st.getCommentPositionEnd();
                        In = true;
                    }
                }else
                if( st.sval.equals("protected") )
                {
                    modifiers.setProtected();
                    if( !In )
                    {
                        position = st.getMiddle();
                        commentPositionB = st.getCommentPositionBegin();
                        if( commentPositionB != null )
                            commentPositionE = st.getCommentPositionEnd();
                        In = true;
                    }
                }else
                if( st.sval.equals("private") )
                {
                    modifiers.setPrivate();
                    if( !In )
                    {
                        position = st.getMiddle();
                        commentPositionB = st.getCommentPositionBegin();
                        if( commentPositionB != null )
                            commentPositionE = st.getCommentPositionEnd();
                        In = true;
                    }
                }else
                if( st.sval.equals("class") )
                {
                    if( !In )
                    {
                        position = st.getMiddle();
                        commentPositionB = st.getCommentPositionBegin();
                        if( commentPositionB != null )
                            commentPositionE = st.getCommentPositionEnd();
                        In = true;
                    }
                    readObject(position,commentPositionB,commentPositionE,
                        pkg,st,xf,modifiers,true,form,loaded);
                    commentPositionB = null;
                    In = false;
    //                position = st.getNextBegin();
                }else
                if( st.sval.equals("interface") )
                {
                    if( !In )
                    {
                        position = st.getMiddle();
                        commentPositionB = st.getCommentPositionBegin();
                        if( commentPositionB != null )
                            commentPositionE = st.getCommentPositionEnd();
                        In = true;
                    }
                    readObject(position,commentPositionB,commentPositionE,
                        pkg,st,xf,modifiers,false,form,loaded);
                    commentPositionB = null;
                    In = false;
  //                  position = st.getNextBegin();
                }
                break;
            default:
                modifiers.reset();
                commentPositionB = null;
                In = false;
//                position = st.getNextBegin();
            }//SWITCH
        }//WHILE
    }
    static void reparsePackage(BrPackage pkg)
        throws FileNotFoundException,IOException
    {
        //System.out.println("Reparse package: ");

        XFile pDir = pkg.getXFile();
        File dir = pDir.getFile();
        if( !dir.isDirectory() )
        {
            //System.out.println("Error 68 in Parser.reparsePackage");
            throw new FileNotFoundException();
        }
        String ss[] = dir.list();

        //System.out.println("Reparse package: " + pDir +
        //    " num " + ss.length);

        File f;
        int pomI,i,j,k = ss.length;
        String javaName = "",pomS = "";

        // co formy???
        for(i=0;i<k;i++)
        {
            //System.out.println(f);
            if( (ss[i] == null) || ((pomI = ss[i].indexOf('.')) < 0) )
            {
                ss[i] = null;
                continue;
            }
            if( form.equalsIgnoreCase(ss[i].substring(pomI)) )
            {
                javaName = (pomS = ss[i].substring(0,pomI)) + java;
                for(j=0;j<k;j++)
                    if( (ss[j] != null) &&
                        (ss[j].equals(javaName) ||
                        ss[j].equals(pomS)) )
                    {
                        ss[j] = null;
                        break;
                    }
                ss[i] = null;
                if( j == k )
                {
                    pkg.addComponent(new BrClass(
                                new MyPosition(0,0),
                                new MyPosition(0,0),
                                null,
                                null,
                                pkg,
                                pomS,
                                new XFile(pDir.getPathStr(),
                                    pDir.getPackage(),pomS),
                                new Modifiers(),
                                "",
                                new String[0],
                                true));

                    continue;
                }
                //f = new File(dir,javaName);
                reparseFile(pkg,new XFile(pDir.getPathStr(),
                    pDir.getPackage(),pomS),true);
            }else
            if( java.equalsIgnoreCase(ss[i].substring(pomI)) )
            {
                ss[i] = ss[i].substring(0,pomI);
            }else
            {
                ss[i] = null;
            }
        }// for

        //               asd.ddf asq.java were.form aasd.java were.java...
        // prevedeno na: null    asq      null      aasd      null
        // .java bez .form
        for(i=0;i<k;i++)
        {
            if( ss[i] == null ) continue;
            f = new File(dir,ss[i] + java);
            //System.out.println("Jde cist.");
            reparseFile(pkg,new XFile(pDir.getPathStr(),
                pDir.getPackage(),ss[i]),false);
        }
        return;
    }
    public static boolean isForm(XFile file)
    {
        String s = file.getFileName();
        int pomI = s.lastIndexOf('.');
        s = s.substring(0,pomI) + form;
        File f = new File(s);
        return f.exists();
    }


    private static Vector sources = new Vector(10,10);
    static void reparseSources()
    {
        sources = new Vector(10,10);
        for(Enumeration e = xelfi.editor.Editor.loadedSources();
            e.hasMoreElements();)
        {
            sources.addElement(e.nextElement());
        }
        return;
    }

    // nasledujici metody se pouzivaji pri vyhledavani vsech
    // package v jistem adr

    /** Prohleda vsechny podadresare pDir, a vytvori prislusne BrPackage.
        Ty pak da do Vectoru ktery vraci...
    */

    static Vector reparseDir(String pDir,int project)
        throws FileNotFoundException
    {
        //System.out.println("Scan directory :" + pDir);
        TopLevel.getOutput().println("Scan directory :" + pDir);

        // Je to opravdu adresar???
        File dir = new File(pDir);
        if( !dir.isDirectory() )
            throw new FileNotFoundException();

        XFile.addPath(pDir);
        Vector v = new Vector(20,10);

        // Dam tam root package
        v.addElement(new BrPackage(new XFile(pDir,"",""),
                    project));
        // A najdu vsechny ostatni.
        reparseDir1(pDir,dir,"",project,v);

        return v;
        //System.out.println("Parser end");
    }

    static String s;
    static private void reparseDir1(String rootPath,File dir,
        String packagee,int project,Vector v)
    {
        //System.out.println(" reparse " + dir);
        String[] ss = dir.list();
        File f;
        int i,k = ss.length;
        for(i=0;i<k;i++)
        {
            f = new File(dir,ss[i]);
            if( f.isDirectory() )
            {
                v.addElement(new BrPackage(new XFile(rootPath,
                    s = packagee + ss[i],""),project));
                reparseDir1(rootPath,f,s + '.',project,v);
            }
        }
        return;
    }
    private BrPackage packagee;
    Parser(BrPackage pkg)
    {
        super("Parser");
        packagee = pkg;
        setPriority(MIN_PRIORITY);
        start();
    }
    public void run()
    {
        try{
            reparsePackage(packagee);
        }catch(FileNotFoundException e)
        {
        }catch(IOException e)
        {
        }
    }

}



/*
na vypis tokenu v souboru...
    while( true )
        {
            try
            {
                ch = st.nextToken();
            }catch( IOException e )
            {
                System.out.println("Chyba pri cteni souboru");
                return;
            }
            try{
                switch( ch )
                {
                case xelfi.browser.document.StreamTokenizer.TT_EOF:
                    os.writeBytes("EOF\n");
                    return;
                case xelfi.browser.document.StreamTokenizer.TT_NUMBER:
                    os.writeBytes("NUMBER " + st.nval + "\n");
                    break;
                case xelfi.browser.document.StreamTokenizer.TT_WORD:
                    os.writeBytes("WORD " + st.sval + "\n");
                    break;
                case xelfi.browser.document.StreamTokenizer.TT_SLASHSTAR:
                    os.writeBytes("SLASHSTAR " + st.sval + "\n");
                    break;
                case xelfi.browser.document.StreamTokenizer.TT_SLASHSLASH:
                    os.writeBytes("SLASHSLASH " + st.sval + "\n");
                    break;
                case xelfi.browser.document.StreamTokenizer.TT_TEXT:
                    os.writeBytes("TEXT " + st.sval + "\n");
                    break;
                default:
                    os.writeBytes("??? " + (char)ch + "\n");
                }
            }catch( IOException e)
            {
                System.out.println("Chyba pri cteni souboru");
                return;
            }

        }*/











