package xelfi.browser.util;

import xelfi.browser.util.Member;

class BTree
{
    public BTree left;
    public BTree right;
    /**
     * This variable contains object in root of this tree.
     */
    public Object member;

    /**
     * This constructor create binary tree with one object in root.
     * @param o object
     * @return binary tree
     */
    public BTree(Object o)
    {
        left = null;
        right = null;
        member = o;
    }
    /**
     * This constructor create empty binary tree.
     * @return binary tree
     */
    public BTree()
    {
        left = null;
        right = null;
        member = null;
    }
    /**
     * This method returns binary tree with specified key
     * of object in root.
     * @param key
     * @return binary tree
     */
    public BTree findMember(String key)
    {
        Member m = (Member)member;
        BTree pom;
        if( m.getKey().equals(key) )
            return this;
        else
        {
            if( (left != null) && (pom = left.findMember(key)) != null )
                return pom;
            else
                if( right != null )
                    return right.findMember(key);
                else
                    return null;
        }
    }
    /**
     * This method adds tree t to the specified place
     * in this tree
     * @param BTree t
     */
    public void toRL(BTree t)
    {
        if( right != null )
            right.toLL(t);
        else
            right = t;
    }
    /**
     * This method adds tree t to the specified place
     * in this tree
     * @param BTree t
     */
    public void toLL(BTree t)
    {
        if( left != null )
            left.toLL(t);
        else
            left = t;
    }
    /**
     * This method adds tree t to the specified place
     * in this tree
     * @param BTree t
     */
    public void toRR(BTree t)
    {
        if( right != null )
            right.toRR(t);
        else
            right = t;
    }
    /**
     * This method adds tree t to the specified place
     * in this tree
     * @param BTree t
     */
    public void toLR(BTree t)
    {
        if( left != null )
            left.toRR(t);
        else
            left = t;
    }
    /**
     * This function converts a binary tree (BTree) an of objects
     * to array of objects and array of depth in the tree
     * @param tree binary tree
     * @param o array of objects
     * @param depth array of depths in tree
     * @return array of depths in tree
     */
    public static int[] toDepthMap(BTree tree,
        Object[] o,int[] depth)
    {
        tree.toDepthMap1(depth,o,0,1);
        return depth;
    }
    private int toDepthMap1(int[] depth,
        Object[] o,int index,int dep)
    {
        depth[index] = dep;
        o[index] = member;

        if( right != null )
            index = right.toDepthMap1(depth,o,index + 1,dep + 1);
        else index++;
        if( left != null )
            return left.toDepthMap1(depth,o,index,dep);
        else
            return index;
    }
    /**
     * This function converts array of Members to the binary tree.
     * Member.getKey function returns parent Member
     * @param v array of Members
     * @return binary tree
     */
    public static BTree toTree(Member[] v)
    {
        int i,k = v.length;
        BTree root = new BTree(v[0]);
        BTree pom = root;
        for(i=1;i<k;i++)
        {
            pom.left = new BTree(v[i]);
            pom = pom.left;
        }

        pom = root;
        BTree last = null,parent = null,pom1;
        do
        {
            parent = root.findMember(((HMember)pom.member).getParent());
            if( parent != null )
            {
                if( last == null )
                {
                    root = pom.left;
                    parent.toRL(pom);
                    pom.left = null;
                    pom = root;
                }else
                {
                    last.left = pom.left;
                    parent.toRL(pom);
                    pom1 = pom.left;
                    pom.left = null;
                    pom = pom1;
                }
            }else
            {
                last = pom;
                pom = pom.left;
            }
        }while( pom != null );

        return root;
    }


}


