/** XelfiCompilerMain class
* @author: Filip Dvorak
* @version: 1.3
*/

package xelfi.compiler;

import java.io.*;
import java.util.Arrays;
import javax.tools.JavaCompiler;
import javax.tools.StandardJavaFileManager;
import javax.tools.ToolProvider;


class XelfiCompilerMain extends Object implements MainCompiler
{
    /** Output from compiler is send here. */
  protected XelfiCompilerOutput out;

  XelfiCompilerMain(XelfiCompilerOutput o)
  {
    out = o;
  }

  public void compileClass(File file)
  {
    String args[] = null;
    if(CompilerOptions.getUseGparam())
    {
      args = new String[2];
      args[0] = "-g";
      args[1] = file.toString();
      output("Compiling: " + args[0] + " " + args[1]);
    }
    else
    {
      args = new String[1];
      args[0] = file.toString();
      output("Compiling: " + args[0]);
    }
    JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
    StandardJavaFileManager fm = compiler.getStandardFileManager(null, null, null);
    try {
      compiler.getTask(new OutputStreamWriter(out), fm, null, Arrays.asList(args).subList(0, args.length - 1), null, fm.getJavaFileObjects(args[args.length - 1])).call();
    } finally {
      try {
        fm.close();
      } catch (IOException ex) {
         ex.printStackTrace();
      }
    }
    output("\n");
  }

  public void error(String err)
  {
    out.topLevelError(err);
  }

  public void output(String msg)
  {
    out.topLevelOutput(msg);
  }
}
