/**
* Xelfi
*/

package xelfi.design.components.palette;

/**
 * @version 	1.61 03 Apr 1997
 * @author		Petr Hamernik
 */

import java.awt.*;
import java.util.Vector;

public class XFPaletteLayout implements LayoutManager {

    private int indent;
    private int minWidth = 0, minHeight = 0;
    private int preferredWidth = 0, preferredHeight = 0;
    private int compW = 0, compH = 0;
    private boolean sizeUnknown = true;

    public XFPaletteLayout(int w, int h) {
        this(4, w, h);
    }

    public XFPaletteLayout(int v, int w, int h) {
        indent = v;
        compW = w;
        compH = h;
    }

    public void addLayoutComponent(String name, Component comp) {
    }

    public void removeLayoutComponent(Component comp) {
    }

    private void setSizes(Container parent) {
        int nComps = parent.countComponents();
        Dimension d = null;

        preferredWidth = 2 * indent + nComps * compW;
        preferredHeight = 2 * indent + compH;
        minWidth = preferredWidth;
        minHeight = preferredHeight;
    }


    public Dimension preferredLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        setSizes(parent);

        Insets insets = parent.insets();
        dim.width = preferredWidth + insets.left + insets.right;
        dim.height = preferredHeight + insets.top + insets.bottom;

        sizeUnknown = false;

        return dim;
    }

    public Dimension minimumLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        Insets insets = parent.insets();
        dim.width = minWidth + insets.left + insets.right;
        dim.height = minHeight + insets.top + insets.bottom;

        sizeUnknown = false;

        return dim;
    }

    public void layoutContainer(Container parent) {
        Insets insets = parent.insets();
        int maxWidth = parent.size().width - (insets.left + insets.right) - 2 * indent;
        int maxHeight = parent.size().height - (insets.top + insets.bottom) - 2 * indent;
        int nComps = parent.countComponents();

        setSizes(parent);

        int maxX = maxWidth / compW;
        int maxY = maxHeight / compH;

		if (maxX <= 0)
			maxX = 1;
        for (int i = 0; i < nComps; i++) {
            Component c = parent.getComponent(i);
            int x = indent + (i % maxX) * compW;
            int y = indent + (i / maxX) * compH;
            if ((i / maxX) > maxY)
                break;
            c.reshape(x, y, compW, compH);
        }
    }

    public String toString() {
        return getClass().getName() + "[indent=" + indent + "]";
    }
}
