package xelfi.design.inspector;

import java.awt.*;
import java.util.*;

import xelfi.design.components.*;
import xelfi.design.forms.*;
import xelfi.util.Debug;

/**
	* XFSelectComponentPanel is a panel which includes choice box for 
	* selecting components which will be visible in CI.
	* 
	* @author Dafe
	* @version 1.01, 3 Dec 1996
	Last change:  DF   29 Jan 97   11:18 pm
*/
class XFSelectComponentPanel extends Panel {

	private XelfiComponentInspector		ci;
	private Choice										compChoice = null;
	private boolean										canStop = true;

	XFSelectComponentPanel (XelfiComponentInspector ci) {
		this.ci = ci;
		setLayout(new GridLayout(1,1));
		changeContent();
	}

	/**
	 * Responds to incoming events
	*/
	public boolean handleEvent (Event e) {
		super.handleEvent(e);
		// debug code
		/*if (e.id==Event.ACTION_EVENT)
      Debug.debug(Debug.INSPECTOR, "Action event catched"); */
		// end of debug
		if ((e.id==Event.ACTION_EVENT) && (e.target==compChoice)) {
			// change in choice - force change of an selection in CI
			Vector newSelection =	new Vector();
			int index = compChoice.getSelectedIndex();
			boolean isHelperLine = compChoice.getItem(0).compareTo("")==0;
			if ((index>0)||(!isHelperLine)) {
				newSelection.addElement(ci.getInspectable().getAllComponents().
																elementAt(isHelperLine ? index-1 : index));
				ci.getInspectable().setSelectedComponents(newSelection);
				ci.selectionChanged();
			}
		}
		return false;
	}

	/**
	 * Empties associated choice box
  */
	void empty () {
    removeAll();
	}

	/**
	 * Changes content of choice box properly
	*/
  void changeContent () {
		// get the inspectable
		XelfiDesignInspectable xi = ci.getInspectable();
		// create new choice with one empty item
		// (will be used for multiple selections)
		compChoice = new Choice();
		int compCount = 0;
		Vector allComps = null;
		Vector selComps = null;
		if (xi!=null)	{
			// get all and selected components
			allComps = xi.getAllComponents();
			selComps = xi.getSelectedComponents();
			compCount = allComps.size();
		}
		if (compCount<=1) {
			// wheee, this is a very bad hack :-(
			compChoice.addItem("");
		}
		if (xi!=null) {
			// fill choice with names and classnames
			for (int i=0; i<allComps.size(); i++) {
				compChoice.addItem(makeItem((XelfiDesignComponent)
														allComps.elementAt(i)));
			}			
			// select an item
			if (selComps.size()==1)
				compChoice.select(makeItem((XelfiDesignComponent)
														selComps.elementAt(0)));
		}
		changeChoice(compChoice);
	}

	/**
	 * Deletes old choice from panel and adds new one with new content.
	 *
	 * Note: This method is synchronized, because thread must not be 
	 * stopped in this method (otherwise it will leave panel in undefined 
	 * state leading to many "lost choices" appear on the screen).
	 * So other thread which wants to stop another one which is using this instance 
	 * should call waitUtilStoppable method!!
	*/
	synchronized void changeChoice (Choice newChoice) {
		// remove old choice
		removeAll();
		// add new choice to panel (disabled or enabled)
		compChoice.enable(compChoice.countItems()>=2);
		add(compChoice);
    // debug code ******
		// Debug.debug(Debug.INSPECTOR, "Comp. count is: "+countComponents());
    // re-layout selection panel
    invalidate();
    validate();
	}

	/**
	 * Sync method. If other thread is doing something in other 
	 * synchronized method of the same instance of this object,
	 * then this method will return AFTER other thread finish its job.
	 * 
	 * Specially here it is used for waiting until other thread do
	 * its job which must not be stopped... (refillChoice method)
	*/
	synchronized void waitUntilStoppable () {
	}

	/**
	 * Makes string representation of given component in choice
	*/
	private String makeItem (XelfiDesignComponent comp) {
		return comp.getName() + ", [" + comp.getClassName() + "]";
	}
}

