/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.Event;
/**
* Any instance of class that implements this interface can be attached to
* Document by calling setActionController
*
* @see Document
*
* @version 1.01
* @author Jaroslav Tulach
*/

public interface ActionController {
  /** This method is called before every insert to any position
  * For multiple inserts (of arrays or Strings) is called only
  * once at begining.
  *
  *
  * @param p the position method insert will insert to
  * @param info optional info sent to insert or null if nothing has been sent
  * @exception NotAllowedException if this insert is not allowed
  */
  public void allowInsert (Position p, Object info) throws NotAllowedException;

  /** This method is called before every delete of single character at any position.
  *
  * @param p the position method delete will delete at
  * @param info optional info sent to delete
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowDelete (Position p, Object info) throws NotAllowedException;

  /** This method is called before every delete of a block (more characters).
  * @param from the first position to delete
  * @param to the last (excluded) position
  * @param info the info
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowMultiDelete (Position from, Position to, Object info)
  throws NotAllowedException;
};



