/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.*;
/** Displays current line number and current column number.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class EditorCounter extends Label implements CursorListener {
  /** Line of the cursor. */
  private int cursor;

  /** the editor */
  private EditorCanvas editor;

  /** Constructs new counter.
  * @param e the editor. Its status wil be displayed
  */
  public EditorCounter (EditorCanvas e) {
    super ("", CENTER);

    setFont (new Font ("Courier", Font.PLAIN, 16));

    (editor = e).addCursorListener (this);

    readCursor ();
    cursorChanged ();
  };

  /** Closes itself. Removes itself from the contoler list.
  */
  public void close () {
    editor.removeCursorListener (this);
  };

  /** Gets the position of cursor from the editor. */
  public void readCursor () {
    try {
      cursor = editor.getDocument ().lineToInt (editor.getCursorPos ());
    } catch (InvalidPositionException ex) {
    };
  };
  
  /** Sets correct values to the display.
  */
  public void cursorChanged () {
    int c = editor.getCursorPos ().getColumn ();
    String line = "    " + String.valueOf (cursor + 1);
    line = line.substring (line.length () - 4);

    String column = (String.valueOf (c + 1) + "    ").substring (0, 4);
    
    setText (line + " : " + column);
  };

  /** Recalculates the size of window.
  * @return the size
  */
  public Dimension preferredSize () {
    Dimension d = super.preferredSize ();
    d.height = 0;
    return d;
  };

  /** Reacts to cursor movement.
  */
  public void cursorMoved (TextEditorInfo info) {
    readCursor ();
    cursorChanged ();
  };

};


