/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.io.File;
import java.io.IOException;
/**
* This interface allows to find a source file if only a class name is specified.
* It should be implemented by someone who maintains class and source paths.
* This interface should be specified to the Editor by call to setSourceFinder method.
* @see Editor#setSourceFinder
*
* @version 1.0
* @author Jaroslav Tulach
*/

public interface SourceFinder {
  /** Finds a source file for the specified class.
  * @param className the name of class to search for
  * @param create If true and the class still do not exist the new source file should be
  * created (only create new file name).
  * @return the source file
  * @exception IOException is thrown either if the class does not exist and create is false
  * or if the class is compiled but sources are unreachable.
  */
  public XelfiFile findSource (String className, boolean create) throws IOException;

  /** Tries to find class name for specified file. This is the reverse process to the
  * findSource funciton.
  * @param f the file
  * @return xelfi file
  * @exception IOException if the file is not valid source file
  */
  public XelfiFile findClass (File f) throws IOException;
};


