/**
* Xelfi programming environment
	Last change:  DF   13 Mar 97    9:51 pm
*/

package xelfi.templates;
import java.util.Hashtable;
import java.util.NoSuchElementException;
import java.util.Enumeration;

import xelfi.awt.XFTree;
import xelfi.awt.XFTreeNode;
/**
* Registers all templates in the system.
*
* @version $Id$
* @autor Jaroslav Tulach
*/

abstract class Registration extends Object {
  /** maps template names to templates */
  private static Hashtable table = new Hashtable ();

  /** table of all TemplateManagers */
  private static Hashtable managers = new Hashtable ();

  /** The tree with all templates.
  */
  private static XFTree tree;

  /** registers basic template to the tree */
  static {
    table.put (Template.basic.name, Template.basic);
		// modified to reflect tree vies change
    // tree = new XFTree (Template.basic);
		tree = new XFTree();
    Template.basic.treeNode = tree.insertInto((Object)Template.basic,
																							tree.getRoot());
  };

  /** Test if the name is registered or not.
  * @param name the name of template
  * @return true if a template with this name is already registered or
  *   the name is not valid and cannot be registered
  */
  static boolean isRegistered (String name) {
    return (table.get (name) != null) || (name.indexOf ('\'') >= 0);
  };
  

  /** Adds new name to the list of templates.
  * If the name is already registered an exception is thrown
  * @param t the template
  * @return the tree node of this template
  * @exception AlreadyUsedException if the name of template is already registrated
  */
  public static XFTreeNode register (Template t) throws AlreadyUsedException {
    synchronized (table) {
      if (isRegistered (t.name)) {
        throw new AlreadyUsedException ();
      } else {
        XFTreeNode tn;
        // add template name to hashtable
        table.put (t.name, t);
        // normal template

        Class manclas = t.manager.getClass ();
        if (managers.get (manclas) == null) {
          // if template's manager has not been registrated yet
          // and it is not the basic manager
          managers.put (manclas, t.manager);
        };

        tn = t.supr.treeNode;
        return t.supr.hasSubTemplates = tn.insert (t, t.supr.hasSubTemplates);
      }
    }
  }

  /** Finds a template only knowing its name.
  * @param name the name of template
  * @return the template
  * @exception NoSuchElementException if no such element can be found
  */
  public static Template find (String name) throws NoSuchElementException {
    Template t = (Template)table.get (name);
    if (t == null) throw new NoSuchElementException ();
    return t;
  };

  /** Returns the tree of all templates
  * @return the tree
  */
  public static XFTree getTree () {
    return tree;
  };

  /** Returns the enumeration of all template managers registered to
  * the system.
  * @return the enumeration of TemplateManager
  */
  public static Enumeration getManagers () {
    return managers.elements ();
  };
};




