/**
* Xelfi programming environment
*/

package xelfi.templates;
import xelfi.editor.EditorLoader;
import xelfi.editor.Source;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.io.IOException;
/**
* This object implements the EditorLoader callback.
* Every time a new source is loaded into memory this
* callback asks every installed TemplateManager if
* it recognizes the source. Then the template is
* set to the document as the default template.
*
* @warning Only one instance of this object should exist.
*
* @version $Id$
* @autor Jaroslav Tulach
*/

class TemplateLoader extends Object implements EditorLoader {
  /** table that maps documents to templates */
  static Hashtable table = new Hashtable ();

  /** This method is called when a new document is loaded into memory.
  * It has to find out who created the source.
  *
  * @param s the source object of the class
  * @exception IOException if an error during identification occurs
  */
  public void newDocumentHasBeenLoaded (Source s) throws IOException {
    // this is hack!!
    // when creating new source by Template.newSource this method is also
    // called but it should not => so return without any action
    if (table.get (s.getName ()) != null) {
      // but first of all test if the document is empty (it must be)
      if (s.getDocument ().isEmpty ()) {
        return;
      } else {
        throw new IOException ("Cannot create new source" + s.getName () + ", because it already exists");
      }
    }

    Template t;
    Enumeration en = Registration.getManagers ();
    try {
      for (;;) {
        TemplateManager tm = (TemplateManager)(en.nextElement ());
        try {
          // can throw IOException
          t = tm.identifySource (s);
          break;
        } catch (NoSuchElementException e) {
          // not identified by this manager
        };
      };
      // used break to get here
      // new value for t has been set
    } catch (NoSuchElementException ex) {
      // no manager wants this
      // use the basic manager
      t = Template.basic;
    };
    // if no IOException then register the source with this template
    table.put (s.getName (), t);
  };

  /** This method is called when an old document is about to close.
  * @param s the source object of the class
  * @warning the source object s will became invalid after finishing this
  * method. So <EM>do not store pointer to it anywhere!</EM>
  * If you do not want to relase the document from memory create new
  * Source object for class s.getName ().
  * But again <EM>do not use the s object itself</EM>.
  */
  public void documentIsAboutToClose (Source s) {
    table.remove (s.getName ());
  };

  /** Finds out the template for a source.
  * @param s the source
  */
  public static Template templateForSource (Source s) {
    return (Template)table.get (s.getName ());
  };
};









