/** OptionsLoader class - it loads and stores all xelfi setting from and to a file
* @author: Filip Dvorak
* @version: 1.2
*/

package xelfi.top;

import java.io.*;
import xelfi.util.*;

  /** Loads and stores options from and into a file. Remebmers file name from which the options were 
    loaded. */
class OptionsLoader
{
    // default name of the config file
  protected static final String DefaultName = ".XelfiOptions";
    // name of file frome which options were loaded or nul if options are from defaults
  protected static File configFile;

    /* This is called from TopLevel. Tries to load the all options from the file, then from the
      current directory, then from the user directory, then loads default.
     */
  static EnhProperties load(String file)
  {
    EnhProperties p = new EnhProperties();

      // cmdline parameter
    if(doLoad(file, p)) return p;

      //current directory
    if(doLoad(DefaultName, p)) return p;

      // user home directory
    String usr = System.getProperty("user.home");
    if(doLoad(PathScanner.addFileSeparator(usr) + DefaultName, p)) return p;

      // using default values
    configFile = null;
    return null;
  }

    /** Called from TopLevel. Stores options into config file. */
  static void store(EnhProperties prop)
  {
    if(configFile == null) configFile = new File(PathScanner.addFileSeparator(System.getProperty("user.dir"))
      + DefaultName);
    try
    {
      FileOutputStream f = new FileOutputStream(configFile.getPath());
      prop.save(f, "The Xelfi IDE configuration file.");
      System.out.println("Saving to " + configFile.getPath());
    }
    catch(IOException e) { System.err.println("Cannot write to " + configFile.getName()); }
  }

  protected static boolean doLoad(String fname, EnhProperties prop)
  {
    try
    {
      if(fname == null) throw new FileNotFoundException();
      FileInputStream f = new FileInputStream(fname);
      prop.load(f);
      File tmp = new File(fname);
      configFile = new File(tmp.getAbsolutePath());
      System.out.println("Config file " + configFile.getPath() + " read.");
    }
    catch(IOException e) { return false; }
    return true;
  }
}


