/** Abstract class for RowLayout and ColLayout managers
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.*;
import java.util.*;

public abstract class LayoutBase implements LayoutManager
{
  protected Vector components, prop;

  public LayoutBase()
  {
    components = new Vector(12);
    prop = new Vector(12);
  }

      /** Not supported method of adding component. */
  public void addLayoutComponent(String  name, Component  comp)
  {}

  public void add(Component comp, LayoutPropertiesBase pr)
  {
    prop.addElement(pr.clone());
    components.addElement(comp);
  }

  public void removeLayoutComponent(Component  comp)
  {
    int ix;
    prop.removeElementAt(ix = components.indexOf(comp));
    components.removeElementAt(ix);
  }

  public abstract void layoutContainer(Container parent);
  public abstract Dimension minimumLayoutSize(Container  parent);
  public abstract Dimension preferredLayoutSize(Container  parent);

    /** Builds an array of preferred sizes. */
  protected Dimension[] prefSizes()
  {
    Component com = null;
    int l = components.size(), i;
    Dimension sizes[] = new Dimension[l];
    for(i = 0; i < l; i++)
    {
      com = (Component)components.elementAt(i);
      sizes[i] = com.preferredSize();
    }
    return sizes;
  }

    /** Builds an array of minimum sizes. */
  protected Dimension[] minSizes()
  {
//    boolean flg = this instanceof RowLayout;

    Component com = null;
    int l = components.size(), i;
    Dimension sizes[] = new Dimension[l];
    for(i = 0; i < l; i++)
    {
      com = (Component)components.elementAt(i);
      sizes[i] = com.minimumSize();
//      if (i == 1 && flg) System.out.println("Komp. c. 2 vraci minheight: " + sizes[i]);
    }
    return sizes;
  }
}
