/** New class (from template) Dialog

* @author: Filip Dvorak

* @version: 1.0

*/



package xelfi.top.visual;



import java.awt.*;
import java.util.Vector;


import xelfi.templates.*;

import xelfi.editor.Source;

import xelfi.top.TopLevel;
import xelfi.util.MessageBox;
import xelfi.browser.document.*;




public class NewClassDialog extends Dialog
{
  List packages;
  TextField clname;
  TemplateView templ;
  Button butOK, butCanc;
  private Vector rootPackageIndexes;
  public NewClassDialog(Frame par)
  {
    super(par, "New class from template", true);
    rootPackageIndexes = new Vector(2);
    packages = new List();
    BrPackage selected[] = ComponentManager.getSelectedPackages();
    if(selected == null || selected.length == 0)
    {
      packages.addItem("<Root>");
      rootPackageIndexes.addElement(new Integer(0));
      // root package
    }
    else
    {
      for(int i = 0; i < selected.length; i++)
      {
        String tmp = selected[i].getXFile().getPackage();
        if(tmp.length() == 0)
        {
          tmp = "<Root>";
          rootPackageIndexes.addElement(new Integer(i));
        }
        packages.addItem(tmp);
      }
    }

      // ratio between template tree and pakage list growth
    float ratio = Math.max(1f, 3f - /* packlist items */ packages.countItems() / 10);
    float packh = 0.77f / (ratio + 1);
    float treeh = 0.77f - packh;
    RowLayout dl = new RowLayout();
    setLayout(dl);
    Insets di1 = new Insets(2, 2, 2, 6);
    Insets di2 = new Insets(0, 8, 2, 6);
    Insets di3 = new Insets(0, 2, 2, 6);
    Insets di4 = new Insets(0, 8, 0, 6);
    RowLayoutProperties dp = new RowLayoutProperties(di1, 0, 0.02f, 0);
    dp.leftSpaceGrowth = dp.rightSpaceGrowth = 0.03f;
      // template tree
    Label tf = new Label("Create from template");
    add(tf);
    dl.add(tf, dp);
    dp.compInsets = di4;
    dp.topSpaceGrowth = 0;
    dp.compGrowth = treeh;
    templ = new TemplateView();
    add(templ);
    dl.add(templ, dp);
    dp.compGrowth = 0;

      // package list
    tf = new Label("Create in package");
    dp.topSpaceGrowth = 0.04f;
    dp.compInsets = di3;
    add(tf);
    dl.add(tf, dp);
    dp.compInsets = di2;
    dp.topSpaceGrowth = 0;
    dp.compGrowth = packh;
    add(packages);
    dl.add(packages, dp);
    dp.compGrowth = 0;

      // class name
    tf = new Label("Class name");
    dp.topSpaceGrowth = 0.04f;
    dp.compInsets = di1;
    add(tf);
    dl.add(tf, dp);
    dp.compInsets = di2;
    dp.topSpaceGrowth = 0;
    clname = new TextField();
    add(clname);
    dl.add(clname, dp);

      // button panel
    Panel bp = new Panel();
    dp.topSpaceGrowth = 0.06f;
    dp.bottomSpaceGrowth = 0.05f;
    dp.compGrowth = 0.02f;
    dp.compInsets = new Insets(6, 2, 3, 2);
    add(bp);
    dl.add(bp, dp);

      // button panel setup
    ColLayout bpl = new ColLayout();
    ColLayoutProperties bpp = new ColLayoutProperties(new Insets(0, 2, 0, 2), 0.02f);
    bpp.leftSpaceGrowth = 0.41f;
    bpp.rightSpaceGrowth = 0.03f;
    butOK = new Button(" OK ");
    bp.add(butOK);
    bpl.add(butOK, bpp);
    bpp.compGrowth = 0.03f;
    bpp.leftSpaceGrowth = 0;
    butCanc = new Button("Cancel");
    bp.add(butCanc);
    bpl.add(butCanc, bpp);
    packages.select(0);

    pack();
  }

  public boolean action(Event e, Object arg)
  {
    if(e.target == butOK)
    {
      newClass();
      return true;
    }
    else if(e.target == butCanc)
    {
      hide();
      dispose();
      return true;
    }
    return super.action(e, arg);
  }

  public void show()
  {
    if(getPeer() == null) addNotify();
    Dimension scr = Toolkit.getDefaultToolkit().getScreenSize(), me = size();

    move((scr.width - me.width) / 2, (int)((scr.height - me.height) / 2.4));
    super.show();
    clname.requestFocus();
  }

  public boolean handleEvent(Event evt)
  {
    switch(evt.id)
    {
      case Event.WINDOW_DESTROY:
        if(evt.target == this)
        {
          hide();
          dispose();
          return true;
        }
        break;
    }
    return super.handleEvent(evt);
  }

  public boolean keyDown(Event  e, int  key)
  {
    if(key == '\n')
    {
      handleEvent(new Event(butOK, Event.ACTION_EVENT, null));
      return true;
    }
    else return false;
  }

  protected void newClass()

  {

    String name = clname.getText();
    int tmp;

    String pack = (tmp = packages.getSelectedIndex()) == -1 ? null :
      (rootPackageIndexes.contains(new Integer(tmp)) ? "" : packages.getSelectedItem());

    Template template = templ.getCurrent();

    if(name.equals("") || template ==  null || pack == null)  return;
    String dot = pack.length() == 0 ? "" : ".";

    try

    {

      Source s = template.newSource(pack + dot + name);

      s.showAll ();
//      TopLevel.showEditor(s, true);

    }

    catch (Exception ex)
    {
      new MessageBox("Error",  "Can't create class " + pack + dot + name +  " (" + ex + ")").show();
      return;
    }

    hide();

    dispose();

  }



  public Dimension preferredSize()

  {

    Dimension pref = super.preferredSize();

    if(pref.width < 260) pref.width = 260;

    if(pref.height > 500) pref.height = 500;

    return pref;

  }



  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.

  {

    Insets i = super.insets();

    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))

      return new Insets(28, 4, 4, 4);

    else return i;

  }

}



