/**
* Xelfi programming environment
*/

package xelfi.util;
/**
* This implementation of BinaryTree
* @see BinaryTree
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class BinaryTreeNodeImpl extends Object implements BinaryTreeNode {
  private Object data;
  private BinaryTreeNode left;
  private BinaryTreeNode right;
  private BinaryTreeNode parent;
  private int value = 0;

  /** empties the object */
  public BinaryTreeNodeImpl () {
    data = left = right = parent = null;
  };

  /** creates new objects and sets data
  * @param o the data to set
  */
  public BinaryTreeNodeImpl (Object o) {
    this ();
    data = o;
  };

  /**
  * @return associated data
  */
  public Object getObject () {
    return data;
  };

  /** sets new data
  * @param b the new data
  */
  public void setObject (Object b) {
    data = b;
  };

  /** @return left son of this node */
  public BinaryTreeNode getLeftSon () {
    return left;
  };

  /** @return right son of this node */
  public BinaryTreeNode getRightSon () {
    return right;
  };

  /** @return parent this node */
  public BinaryTreeNode getParent () {
    return parent;
  };

  /** Sets new left son of this node
  * @param n the new value to set
  */
  public void setLeftSon (BinaryTreeNode n) {
    left = n;
  };

  /** Sets new right son of this node
  * @param n the new value to set
  */
  public void setRightSon (BinaryTreeNode n) {
    right = n;
  };

  /** Sets new parent of this node
  * @param n the new value to set
  */
  public void setParent (BinaryTreeNode n) {
    parent = n;
  };

  /** Gets value from the node.
  * @return the value stored in this node
  */
  public int getValue () {
    return value;
  };

  /** Sets new value to the node.
  * @param val the value to set
  */
  public void setValue (int val) {
    value = val;
  };

  /** converts to string
  */
  public String toString () {
    return "Node with value " + getValue () + ": " + (data == null ? "null" : data.toString ()) ;
  };

};




