/**
* Xelfi programming environment
*/

package xelfi.xedit;
import java.awt.*;
import xelfi.editor.*;
import xelfi.util.MessageBox;
/**
* The dialog for find.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class DialogFind extends Dialog {
  TextField textFind;
  TextField textReplace;

  Button find;
  Button replace;
  Button cancel;
  Checkbox sensitive;
  TextEditor editor;
  Block found;

  /** Creates new find dialog.
  * @param f the parent frame
  * @param e the editor to search in
  */
  DialogFind (Frame f, TextEditor e) {
    super (f, "Find and replace", false);
    editor = e;

    Panel p = new Panel ();
    p.setLayout (new GridLayout (4, 1));
    p.add (new Label ("Text to find:"));
    p.add (textFind = new TextField ());
    p.add (new Label ("Replace with:"));
    p.add (textReplace = new TextField ());
    p.add (sensitive = new Checkbox ("Case sensitive search"));

    add ("Center", p);

    p = new Panel ();
    p.setLayout (new FlowLayout ());
    p.add (find = new Button ("Find"));
    p.add (replace = new Button ("Replace and find"));
    p.add (cancel = new Button ("Cancel"));
    add ("South", p);

    replace.disable ();
    textFind.requestFocus ();

    pack ();
  }

  /** Shows this dialogs and sets focus to the find input line
  */
  public void show () {
    super.show ();
    textFind.requestFocus ();
  }

  /** Handle event.
  */
  public boolean handleEvent (Event ev) {
    if (!super.handleEvent (ev)) {
      if (ev.id == Event.WINDOW_DESTROY) {
        hide ();
        return true;
      }
      return false;
    } else {
      return true;
    }
  }

  public boolean action (Event ev, Object w) {
    if (ev.target == cancel) {
      replace.disable ();
      hide ();
      return true;
    }

    if (ev.target == sensitive) {
      return true;
    }

    try {
      if (ev.target == replace) {
        found.set (textReplace.getText ());
      }

      Position p = editor.getCursorPos ();
      if (found != null) p = p.next ();
      p = EditorOperation.findWord (
        p,
        textFind.getText (),
        sensitive.getState (),
        true
      );
      editor.setCursorPos (p);
      try {
        Position beg = p.previous (textFind.getText ().length ());
        editor.setSelectedBlock (Block.between (beg, p));
        editor.setCursorPos (p);
        found = new Block (beg, p);
        replace.enable ();
      } catch (InvalidPositionException ex) {
      }
      return true;
    } catch (EndOfTextException ex) {
      replace.disable ();
      new MessageBox ("Warning", "Text \"" + textFind.getText () + "\" has not been found").show ();
    } catch (NotAllowedException e) {
    }

    return true;
  }
}




