/**
* Xelfi programming environment
*/

package xelfi.xedit;
import java.awt.*;
import xelfi.editor.*;
/**
* The dialog for go to line.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class DialogGoToLine extends Dialog {
  TextField line;
  Button cancel;
  TextEditor editor;

  /** Creates new go to line dialog.
  * @param f the parent frame
  * @param e the editor to go to line in
  */
  public DialogGoToLine (Frame f, TextEditor e) {
    super (f, "Go to line", false);
    editor = e;

    add ("Center", line = new TextField (4));
    add ("North", new Label ("Enter the line number:"));

    Panel p = new Panel ();
    p.setLayout (new FlowLayout ());
    p.add (new Button ("OK"));
    p.add (cancel = new Button ("Cancel"));
    add ("South", p);

    line.requestFocus ();
  }

  /** Shows this dialogs and sets focus to the find input line
  */
  public void show () {
    super.show ();
    line.requestFocus ();
  }
  /** Handle event.
  */
  public boolean handleEvent (Event ev) {
    if (!super.handleEvent (ev)) {
      if (ev.id == Event.WINDOW_DESTROY) {
        hide ();
        return true;
      }
      return false;
    } else {
      return true;
    }
  }
  
  public boolean action (Event ev, Object w) {
    if (ev.target == cancel) {
      hide ();
      return true;
    }

    try {
      line.selectAll ();
      Integer i = Integer.valueOf (line.getText ());
      try {
        Position p = editor.getDocument ().goToLine (i.intValue () - 1);
        editor.setCursorPos (p);
      } catch (EndOfTextException ex) {
        new xelfi.util.MessageBox ("Warning", "Line " + i + " does not exists").show ();
      }
    } catch (NumberFormatException ex) {
      new xelfi.util.MessageBox ("Warning", "\"" + line.getText () + "\"is not number").show ();
    }
    return true;
  }
}




