/**
* Xelfi programming environment
*/

package xelfi.xedit;
import xelfi.top.XelfiOptions;
import xelfi.util.EnhProperties;
import xelfi.editor.Editor;
import xelfi.editor.StandardEditorCallback;
import java.awt.*;
/**
* Options for editor.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class EditorOptions extends Panel implements XelfiOptions {
  /** the editor */
  TextField tabSize;

  /** backup files */
  Checkbox backup;

  /** preferred width */
  TextField width;

  /** preferred height */
  TextField height;

  /** Font size */
  TextField fontSize;

  /** Constructs new panel
  */
  public EditorOptions () {
    Panel p = new Panel ();
    p.setLayout (new GridLayout (11, 1));

    p.add (new Label ("Size of TAB:"));
    tabSize = new TextField ();
    p.add (tabSize);
    backup = new Checkbox ("Backup files?");
    p.add (backup);
    p.add (new Label ("Preferred width of text editor window"));
    width = new TextField ();
    p.add (width);
    p.add (new Label ("Preferred height of text editor window"));
    height = new TextField ();
    p.add (height);
/*
    p.add (new Label ("Font size"));
    fontSize = new TextField ();
    p.add (fontSize);
*/
    add ("Center", p);
  };

  /** Loads properties for editor.
  * @param src the properties to load from
  */
  public void loadProperties(EnhProperties src) {
    Editor.tabSize = src.getIntProperty ("xelfi.editor.tabsize", 4);
    Editor.backupFiles = src.getBooleanProperty ("xelfi.editor.backup", true);
    Editor.preferredWidth = src.getIntProperty ("xelfi.editor.preferredWidth", 80);
    Editor.preferredHeight = src.getIntProperty ("xelfi.editor.preferredHeight", 20);
    Font f = StandardEditorCallback.font;
    /*
    String name = src.getProperty ("xelfi.editor.font.name", "Courier");
    int style = src.getIntProperty ("xelfi.editor.font.style", Font.PLAIN);
    int size = src.getIntProperty ("xelfi.editor.font.size", 14);
    SyntaxComments.setFont (new Font (name, style, size));
    */
  };

  /** Save your values into dest.
  * @param dest the properties to store to
  */
  public void storeProperties(EnhProperties dest) {
    dest.setIntProperty ("xelfi.editor.tabsize", Editor.tabSize);
    dest.setBooleanProperty ("xelfi.editor.backup", Editor.backupFiles);
    dest.setIntProperty ("xelfi.editor.preferredHeight", Editor.preferredHeight);
    dest.setIntProperty ("xelfi.editor.preferredWidth", Editor.preferredWidth);
    /*
    Font f = StandardEditorCallback.font;
    dest.put ("xelfi.editor.font.name", f.getName ());
    dest.setIntProperty ("xelfi.editor.font.style", f.getStyle ());
    dest.setIntProperty ("xelfi.editor.font.size", f.getSize ());
    */
  };

  /** The panel for options.
  * @return the panel
  */
  public Panel getPanel() {
    tabSize.setText (String.valueOf (Editor.tabSize));
    backup.setState (Editor.backupFiles);
    width.setText (String.valueOf (Editor.preferredWidth));
    height.setText (String.valueOf (Editor.preferredHeight));
    /*
    fontSize.setText (String.valueOf (StandardEditorCallback.font.getSize ()));
    */
    return this;
  };

  /** Name of the tab.
  * @return the name of tab
  */
  public String getTabName() {
    return "Editor";
  };

  /** Saves values from p.
  * @param p the panel
  */
  public void onOK(Panel p) {
    try {
      Integer i = Integer.valueOf (tabSize.getText ());
      Editor.tabSize = i.intValue ();
    } catch (NumberFormatException ex) {
    };
    Editor.backupFiles = backup.getState ();
    try {
      Integer i = Integer.valueOf (width.getText ());
      Editor.preferredWidth = i.intValue ();
    } catch (NumberFormatException ex) {
    };
    try {
      Integer i = Integer.valueOf (height.getText ());
      Editor.preferredHeight = i.intValue ();
    } catch (NumberFormatException ex) {
    };
    /*
    try {
      Integer i = Integer.valueOf (fontSize.getText ());
      int size = i.intValue ();
      Font f = StandardEditorCallback.font;
      f = new Font (f.getName (), f.getStyle (), size);
      StandardEditorCallback.font = f;
    } catch (NumberFormatException ex) {
    };
    */
  };
};















